#include "fmc.h"
#include "Sdram.h"
#include "qspi_w25q64.h"
#include "Tests.h"
#include "debug.h"
#include <stdint.h>

void BoardTest() {

    UART1_Test();
   
    SDRAM_Initialization_Sequence(&hsdram1);
    SDRAM_Test();

  
   QSPI_W25Qxx_Init();
   QSPI_Test();
    
  
}


ErrorStatus SDRAM_Test(void)
{
    uint32_t i = 0;        
    uint16_t ReadData = 0; 
    uint8_t  ReadData8 = 0;
    uint32_t ExecutionTime_Begin; 
    uint32_t ExecutionTime_End;  
    uint32_t ExecutionTime;       
    float ExecutionSpeed;         

    DBG_log("\r\n -------------------- SDRAM 16 bit access -------------------\r\n"); 
    
    // -------------------- write ----------------------------- 
    ExecutionTime_Begin = HAL_GetTick(); // Start time

    for (i = 0; i < SDRAM_Size / 2; i++)
    {
        *(__IO uint16_t *)(SDRAM_BANK_ADDR + 2 * i) = (uint16_t)i; // Write data to SDRAM
    }
    ExecutionTime_End = HAL_GetTick();                                       // End time
    ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
    ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S

    DBG_log("\r\nWrite 16-bit data , size: %d MB, elapsed time: %d ms, write speed: %.2f MB/s\r\n", SDRAM_Size / 1024 / 1024, ExecutionTime, ExecutionSpeed);

    // ----------------------------- read -------------------------------
    ExecutionTime_Begin = HAL_GetTick(); // Start time

    for (i = 0; i < SDRAM_Size / 2; i++)
    {
        ReadData = *(__IO uint16_t *)(SDRAM_BANK_ADDR + 2 * i); // Read data from SDRAM
    }

    ExecutionTime_End = HAL_GetTick();                                       // End time
    ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
    ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S

    DBG_log("\r\nRead 16-bit data, size: %d MB, elapsed time: %d ms, read speed: %.2f MB/s\r\n", SDRAM_Size / 1024 / 1024, ExecutionTime, ExecutionSpeed);


    DBG_log("\r\n -------------------- 16 Bit data validation  -------------------\r\n"); 
 
    for (i = 0; i < SDRAM_Size / 2; i++)
    {
        ReadData = *(__IO uint16_t *)(SDRAM_BANK_ADDR + 2 * i); // Read data from SDRAM
        if (ReadData != (uint16_t)i)                            // Detect the data, if not equal, jump out of the function, return to the detection failure results.
        {
            DBG_log("\r\nSDRAM 16 Bit data validation failed!\r\n");
            return ERROR; //Returns the failure flag
        }
    }

    DBG_log("\r\nSDRAM 16-bit data validation pass!\r\n");
    
    DBG_log("\r\n -------------------- SDRAM 8 bit access  -------------------\r\n"); 
    
    // ------------------ write ----------------------------------
    ExecutionTime_Begin = HAL_GetTick(); // Start time

    for (i = 0; i < SDRAM_Size; i++)
    {
           *(__IO uint8_t *)(SDRAM_BANK_ADDR + i) = (uint8_t)i;
    }

    ExecutionTime_End = HAL_GetTick();                                       // End time
    ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
    ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S
    DBG_log("\r\nWrite 8-bit data, size: %d MB, elapsed time: %d ms, write speed: %.2f MB/s\r\n", SDRAM_Size / 1024 / 1024, ExecutionTime, ExecutionSpeed);

    // ---------------------- read ---------------------------------------------

    ExecutionTime_Begin = HAL_GetTick(); // Start time
    for (i = 0; i < SDRAM_Size; i++)
    {
        ReadData8 = *(__IO uint8_t *)(SDRAM_BANK_ADDR + i);
    }
  

    ExecutionTime_End = HAL_GetTick();                                       // End time
    ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
    ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S
    DBG_log("\r\nRead 8-bit data, size: %d MB, elapsed time: %d ms, read speed: %.2f MB/s\r\n", SDRAM_Size / 1024 / 1024, ExecutionTime, ExecutionSpeed);

   //----------------------- validate data -------------------------------

     for (i = 0; i < SDRAM_Size; i++)
    {
        ReadData8 = *(__IO uint8_t *)(SDRAM_BANK_ADDR + i);
        if (ReadData8 != (uint8_t)i) // Detect the data, if not equal, jump out of the function, return to the detection failure results.
        {
            DBG_log("\r\nSDRAM 8-Bit data validation failed!\r\n");
            return ERROR;
        }
    }
    
    DBG_log("\r\nSDRAM 8-Bit data validation pass!\r\n");


   return SUCCESS; 

}

/********************************************************************************************************
 * 
 *  QSPI_Test 
 * 
 * 
 * 
 *******************************************************************************************************
 */

#define W25Qxx_NumByteToTest 32 * 1024
void QSPI_Test() {

    int32_t QSPI_Status = QSPI_W25Qxx_OK;
    uint32_t i = 0;
    uint32_t W25Qxx_TestAddr = 0;
    uint8_t  W25Qxx_WriteBuffer[W25Qxx_NumByteToTest];  
    uint8_t  W25Qxx_ReadBuffer[W25Qxx_NumByteToTest];

    uint32_t ExecutionTime_Begin; 
    uint32_t ExecutionTime_End;  
    uint32_t ExecutionTime;       
    float ExecutionSpeed;         

    DBG_log("\r\n -------------------- QSPI Test -------------------\r\n"); 
	  
    // -------------------- Block erase ----------------------------- 
    ExecutionTime_Begin = HAL_GetTick(); // Start time

    QSPI_Status = QSPI_W25Qxx_BlockErase_32K(W25Qxx_TestAddr);
       
    ExecutionTime_End = HAL_GetTick();                                       // End time
    ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
    ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S

    if (QSPI_Status == QSPI_W25Qxx_OK)
    {
        DBG_log("\r\nQSPI Erase 32k Block, %d ms\r\n", ExecutionTime);       // Log time
    }
    else
    {
        DBG_log("\r\nErrror QSPI Erase 32k Block, %d \r\n", QSPI_Status);    // log error
    }

    // -------------------- Write buffer  ----------------------------- 
    if (QSPI_Status == QSPI_W25Qxx_OK ) // When previous tests OK
    {
        for (i = 0; i < W25Qxx_NumByteToTest; i++) // Fill write buffer first
        {
            W25Qxx_WriteBuffer[i] = i;
        }
        
        ExecutionTime_Begin = HAL_GetTick(); // Start time

        QSPI_Status = QSPI_W25Qxx_WriteBuffer(W25Qxx_WriteBuffer, W25Qxx_TestAddr, W25Qxx_NumByteToTest); // write buffer
        
        ExecutionTime_End = HAL_GetTick();                                       // End time
        ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
        ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S

        if (QSPI_Status == QSPI_W25Qxx_OK)
        {   
            // if OK log time
            DBG_log("\r\n\r\n! Due to the short times and the ms resolution, error is very high");
            DBG_log("\r\nQSPI Write Buffer, size: %d KB, elapsed time: %d ms, speed: %.2f KB/s\r\n", W25Qxx_NumByteToTest / 1024, ExecutionTime, ExecutionSpeed);
        }
        else
        {   
            // log error 
            DBG_log("\r\nErrror QSPI Write Buffer, %d \r\n", QSPI_Status);    
        }

    }
    
    // -------------------- Read buffer  ----------------------------- 
    if (QSPI_Status == QSPI_W25Qxx_OK ) // When previous tests OK
    {

        ExecutionTime_Begin = HAL_GetTick(); // Start time

        QSPI_Status = QSPI_W25Qxx_ReadBuffer(W25Qxx_ReadBuffer, W25Qxx_TestAddr, W25Qxx_NumByteToTest); // read buffer
        
        ExecutionTime_End = HAL_GetTick();                                       // End time
        ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
        ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S

        if (QSPI_Status == QSPI_W25Qxx_OK)
        {   
            // if OK log time
            DBG_log("\r\n\r\n! Due to the short times and the ms resolution, error is very high");
            DBG_log("\r\nQSPI Read Buffer, size: %d KB, elapsed time: %d ms, speed: %.2f KB/s\r\n", W25Qxx_NumByteToTest / 1024, ExecutionTime, ExecutionSpeed);
        }
        else
        {   
            // log error 
            DBG_log("\r\nErrror QSPI Read Buffer, %d \r\n", QSPI_Status);   
        }

    }

    // -------------------- Validate Read and Write Buffer are indentical  ----------------------------- 
    if (QSPI_Status == QSPI_W25Qxx_OK ) // When previous tests OK
    {
       
        for (i = 0; i < W25Qxx_NumByteToTest; i++)
        {
            
            QSPI_Status  = QSPI_W25Qxx_OK;
            if (W25Qxx_WriteBuffer[i] != W25Qxx_ReadBuffer[i]) 
            {
                QSPI_Status  = W25Qxx_ERROR_DataNotValid; 
                DBG_log("\r\nERROR QSPI Read and Write Buffer not identical\r\n");
                break;
            }
        }    
         
        if ( QSPI_Status  == QSPI_W25Qxx_OK )
        {
                DBG_log("\r\nPASS QSPI Read and Write Buffer are identical\r\n");
        }
    }

    // ---------------------------- Read entire Flash to test the speed -------------------------
    if (QSPI_Status == QSPI_W25Qxx_OK ) // When previous tests OK
    {
        ExecutionTime_Begin = HAL_GetTick(); // Start time

        for (i = 0; i < W25Qxx_FlashSize / (W25Qxx_NumByteToTest); i++) // Read entire Flash 
        {
            // Read W25Qxx_NumByteToTest 
            QSPI_Status = QSPI_W25Qxx_ReadBuffer(W25Qxx_ReadBuffer, W25Qxx_TestAddr, W25Qxx_NumByteToTest);
            W25Qxx_TestAddr = W25Qxx_TestAddr + W25Qxx_NumByteToTest;
        }        

        ExecutionTime_End = HAL_GetTick();                                       // End time
        ExecutionTime = ExecutionTime_End - ExecutionTime_Begin;                 // Elapsed time
        ExecutionSpeed = (float)SDRAM_Size / 1024 / 1024 / ExecutionTime * 1000; // Speed MB/S

        if (QSPI_Status == QSPI_W25Qxx_OK)
        {   
            // if OK log time
            DBG_log("\r\nQSPI Read entire Flash, size: %d KB, elapsed time: %d ms, speed: %.2f MB/s\r\n", W25Qxx_FlashSize / 1024 / 1024, ExecutionTime, ExecutionSpeed);
        }
        else
        {   
            // log error 
            DBG_log("\r\nErrror QSPI Read entire Flash, %d \r\n", QSPI_Status);   
        }
    }

}



